#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <syslog.h>
#include <sys/ioctl.h>
#include <net/if.h>

#define ENABLE_DEBUG
#define SET_TZ
#define NO_DELAY	/* yuecheng */

/* dungjiun add for buffalo WBMR-V2, restart processing time = (Connect on Demand: Max Idle Time) + 10 Min */
#define NTP_RESTART_PROCESS_TIME    1

extern char *optarg;

#ifdef linux
#include <sys/utsname.h>
#include <sys/time.h>
typedef u_int32_t __u32;
#include <sys/timex.h>
#else
#define main ntpclient
extern struct hostent *gethostbyname(const char *name);
extern int h_errno;
#define herror(hostname) \
	fprintf(stderr,"Error %d looking up hostname %s\n", h_errno,hostname)
typedef uint32_t __u32;
#endif

#define JAN_1970        0x83aa7e80      /* 2208988800 1970 - 1900 in seconds */
#define NTP_PORT (123)
#define DAY_TIME 86400

/* How to multiply by 4294.967296 quickly (and not quite exactly)
 * without using floating point or greater than 32-bit integers.
 * If you want to fix the last 12 microseconds of error, add in
 * (2911*(x))>>28)
 */
#define NTPFRAC(x) ( 4294*(x) + ( (1981*(x))>>11 ) )

/* The reverse of the above, needed if we want to set our microsecond
 * clock (via settimeofday) based on the incoming time in NTP format.
 * Basically exact.
 */
#define USEC(x) ( ( (x) >> 12 ) - 759 * ( ( ( (x) >> 10 ) + 32768 ) >> 16 ) )

/* Converts NTP delay and dispersion, apparently in seconds scaled
 * by 65536, to microseconds.  RFC1305 states this time is in seconds,
 * doesn't mention the scaling.
 * Should somehow be the same as 1000000 * x / 65536
 */
#define sec2u(x) ( (x) * 15.2587890625 )

struct ntptime {
	unsigned int coarse;
	unsigned int fine;
};

void send_packet(int usd, struct ntptime *udp_send_ntp);
int rfc1305print(char *data, struct ntptime *arrival, struct ntptime *udp_send_ntp);
int udp_handle(int usd, char *data, int data_len, struct sockaddr *sa_source, int sa_len, struct ntptime *udp_send_ntp);

/* global variables (I know, bad form, but this is a short program) */
char incoming[1500];
struct timeval time_of_send;
int set_clock=0;   /* non-zero presumably needs root privs */

#define TZ_OPTION	"z:"
#define DST_OPTION	"x:"
#define INTERVAL_OPTION	"t:"

static int tz_value = 46;		/* Taipei GMT+8:00*/
static int dst_enabled = 0;
static int update_interval = DAY_TIME;

#ifdef ENABLE_DEBUG
int debug=0;
#define DEBUG_OPTION "d"
#else
#define debug 0
#define DEBUG_OPTION
#endif

/* yuecheng */
#ifdef NO_DELAY
int no_delay=0;
#define NO_DELAY_OPTION "u"
#else
#define no_delay 0
#define NO_DELAY_OPTION
#endif

static void do_DLSTime();

int contemplate_data(unsigned int absolute, double skew, double errorbar, int freq);

#define nvram_safe_get(name) (nvram_get(name) ? : "")

int get_current_freq()
{
	/* OS dependent routine to get the current value of clock frequency.
	 */
#ifdef linux
	struct timex txc;
	txc.modes = 0;
	//if (__adjtimex(&txc) < 0) { //yuecheng
	if (adjtimex(&txc) < 0) {
		perror("adjtimex"); exit(1);
	}
	return txc.freq;
#else
	return 0;
#endif
}

void send_packet(int usd, struct ntptime *udp_send_ntp)
{
	__u32 data[12];
	struct timeval now;
#define LI 0
#define VN 3
#define MODE 3
#define STRATUM 0
#define POLL 4 
#define PREC -6

	if (debug) fprintf(stderr,"Sending ...\n");
	if (sizeof(data) != 48) {
		fprintf(stderr,"size error\n");
		return;
	}
	bzero((char*)data,sizeof(data));
	data[0] = htonl (
		( LI << 30 ) | ( VN << 27 ) | ( MODE << 24 ) |
		( STRATUM << 16) | ( POLL << 8 ) | ( PREC & 0xff ) );
	data[1] = htonl(1<<16);  /* Root Delay (seconds) */
	data[2] = htonl(1<<16);  /* Root Dispersion (seconds) */
	gettimeofday(&now,NULL);
	data[10] = htonl(now.tv_sec + JAN_1970); /* Transmit Timestamp coarse */
	data[11] = htonl(NTPFRAC(now.tv_usec));  /* Transmit Timestamp fine   */
	
	udp_send_ntp->coarse = now.tv_sec + JAN_1970;
	udp_send_ntp->fine   = NTPFRAC(now.tv_usec);
	
	send(usd, data, 48, 0);
	time_of_send = now;
}

int udp_handle(int usd, char *data, int data_len, 
				  struct sockaddr *sa_source, int sa_len, struct ntptime *udp_send_ntp)
{
	struct timeval udp_arrival;
	struct ntptime udp_arrival_ntp;
	int ret=1;
    time_t now;
	struct tm *mytm;
    char time_buf[50];

#ifdef _PRECISION_SIOCGSTAMP
	if ( ioctl(usd, SIOCGSTAMP, &udp_arrival) < 0 ) {
		perror("ioctl-SIOCGSTAMP");
		gettimeofday(&udp_arrival, NULL);
	}
#else
	gettimeofday(&udp_arrival, NULL);
#endif
	udp_arrival_ntp.coarse = udp_arrival.tv_sec + JAN_1970;
	udp_arrival_ntp.fine = NTPFRAC(udp_arrival.tv_usec);
	
	if ((udp_arrival_ntp.coarse > (udp_send_ntp->coarse + 5)) 
		||((udp_arrival_ntp.coarse == (udp_send_ntp->coarse + 5))&&(udp_arrival_ntp.fine > udp_send_ntp->fine)))
		return -1;

	if (debug) {
		struct sockaddr_in *sa_in = (struct sockaddr_in *)sa_source;

		printf("packet of length %d received\n", data_len);
		if (sa_source->sa_family == AF_INET) {
			printf("Source: INET Port %d host %s\n",
			ntohs(sa_in->sin_port), inet_ntoa(sa_in->sin_addr));
		} else {
			printf("Source: Address family %d\n", sa_source->sa_family);
		}
	}
        //kenny +
	{
#define NTP_STATUS	"/tmp/NTP_connect"
		FILE *fp;
	        fp = fopen(NTP_STATUS, "w+");
        	fprintf(fp, "connected");
	        fclose(fp);
	}
        //end of add
	ret = rfc1305print(data, &udp_arrival_ntp, udp_send_ntp);
	do_DLSTime();

    time(&now);
	mytm = localtime(&now);
    strftime(time_buf, 50, "%a %b %d %H:%M:%S %Y", mytm);

	syslog(6, "[NTP SUCCESS: set time : %s]", time_buf);
	//system("/etc/rc.d/cmdsched.sh restart both");
	system("/etc/rc.d/ntp_success.sh");
	return ret;

}

double ntpdiff( struct ntptime *start, struct ntptime *stop)
{
	int a;
	unsigned int b;
	a = stop->coarse - start->coarse;
	if (stop->fine >= start->fine) {
		b = stop->fine - start->fine;
	} else {
		b = start->fine - stop->fine;
		b = ~b;
		a -= 1;
	}
	
	return a*1.e6 + b * (1.e6/4294967296.0);
}

int rfc1305print(char *data, struct ntptime *arrival, struct ntptime *udp_send_ntp)
{
/* straight out of RFC-1305 Appendix A */
	int li, vn, mode, stratum, poll, prec;
	int delay, disp, refid;
	struct ntptime reftime, orgtime, rectime, xmttime;
	double etime,stime,skew1,skew2;
	int freq;

#define Data(i) ntohl(((unsigned int *)data)[i])
	li      = Data(0) >> 30 & 0x03;
	vn      = Data(0) >> 27 & 0x07;
	mode    = Data(0) >> 24 & 0x07;
	stratum = Data(0) >> 16 & 0xff;
	poll    = Data(0) >>  8 & 0xff;
	prec    = Data(0)       & 0xff;
	if (prec & 0x80) prec|=0xffffff00;
	delay   = Data(1);
	disp    = Data(2);
	refid   = Data(3);
	reftime.coarse = Data(4);
	reftime.fine   = Data(5);
	orgtime.coarse = Data(6);
	orgtime.fine   = Data(7);
	rectime.coarse = Data(8);
	rectime.fine   = Data(9);
	xmttime.coarse = Data(10);
	xmttime.fine   = Data(11);
#undef Data

	if ((orgtime.coarse != udp_send_ntp->coarse)
		|| ((orgtime.coarse == udp_send_ntp->coarse) && (orgtime.fine != udp_send_ntp->fine))) {
		if (debug) {
			printf("The Originate Timestamp in the server reply");
			printf("doesn't match the Transmit Timestamp used in the request\n");
		}
		return -1;
	}
	if ((xmttime.coarse == 0) && (xmttime.fine == 0)) {
		if (debug) {
			printf("The Transmit Timestamp in the server reply is zero\n");
		}
		return -1;
	}
	if (mode != 4) {
		if (debug) {
			printf("The Mode in the server reply is not 4 (unicast)\n");
		}
		return -1;
	}

	if (set_clock) {   /* you'd better be root, or ntpclient will crash! */
		struct timeval tv_set;
		/* it would be even better to subtract half the slop */
		tv_set.tv_sec  = xmttime.coarse - JAN_1970;
		/* divide xmttime.fine by 4294.967296 */
		tv_set.tv_usec = USEC(xmttime.fine);
		
		if (settimeofday(&tv_set,NULL) < 0) {
			perror("settimeofday");
			exit(1);
		}
		if (debug) {
			printf("set time to %lu.%.6lu\n", tv_set.tv_sec, tv_set.tv_usec);
		}
	}

	if (debug) {
		printf("LI=%d  VN=%d  Mode=%d  Stratum=%d  Poll=%d  Precision=%d\n",
			li, vn, mode, stratum, poll, prec);
		printf("Delay=%.1f  Dispersion=%.1f  Refid=%u.%u.%u.%u\n",
			sec2u(delay),sec2u(disp),
			refid>>24&0xff, refid>>16&0xff, refid>>8&0xff, refid&0xff);
		printf("Reference %u.%.10u\n", reftime.coarse, reftime.fine);
		printf("Originate %u.%.10u\n", orgtime.coarse, orgtime.fine);
		printf("Receive   %u.%.10u\n", rectime.coarse, rectime.fine);
		printf("Transmit  %u.%.10u\n", xmttime.coarse, xmttime.fine);
		printf("Our recv  %u.%.10u\n", arrival->coarse, arrival->fine);
	}
	
	etime = ntpdiff(&orgtime, arrival);
	stime = ntpdiff(&rectime, &xmttime);
	skew1 = ntpdiff(&orgtime, &rectime);
	skew2 = ntpdiff(&xmttime, arrival);
	freq = get_current_freq();
	
	if (debug) {
		printf("Total elapsed: %9.2f\n"
	       		"Server stall:  %9.2f\n"
	      		"Slop:          %9.2f\n",
			etime, stime, etime-stime);
		printf("Skew:          %9.2f\n"
	       		"Frequency:     %9d\n"
	       		" day   second     elapsed    stall     skew  dispersion  freq\n",
			(skew1-skew2)/2, freq);
		printf("%d %5d.%.3d  %8.1f %8.1f  %8.1f %8.1f %9d\n",
			arrival->coarse/86400+15020, arrival->coarse%86400,
			arrival->fine/4294967, etime, stime,
			(skew1-skew2)/2, sec2u(disp), freq);
	}
	
	return 1;
}


int stuff_net_addr(struct in_addr *p, char *hostname)
{
	struct hostent *ntpserver;
	
	ntpserver = gethostbyname(hostname);
	if (ntpserver == NULL) {
		herror(hostname);
		return 0;
	}
	if (ntpserver->h_length != 4) {
		fprintf(stderr, "oops %d\n", ntpserver->h_length);
		return 0;
	}
	
	memcpy(&(p->s_addr), ntpserver->h_addr_list[0], 4);
	return 1;
}

int setup_receive(int usd, unsigned int interface, short port)
{
	struct sockaddr_in sa_rcvr;
	int i=1;
	
	bzero((char *) &sa_rcvr, sizeof(sa_rcvr));
	sa_rcvr.sin_family = AF_INET;
	sa_rcvr.sin_addr.s_addr = htonl(interface);
	sa_rcvr.sin_port = htons(port);

	if (setsockopt(usd, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)) == -1) {
		fprintf(stderr, "setsockopt(ntp, SO_REUSEADDR): %m");
	}	
	if(bind(usd, (struct sockaddr *) &sa_rcvr, sizeof(sa_rcvr)) == -1) {
		fprintf(stderr, "Could not bind to udp port %d\n", port);
		perror("bind");
		return 0;
	}
	
	listen(usd, 3);
	return 1;
}

int setup_transmit(int usd, char *host, short port)
{
	struct sockaddr_in sa_dest;
	
	bzero((char *) &sa_dest, sizeof(sa_dest));
	sa_dest.sin_family = AF_INET;
	
	if (!stuff_net_addr(&(sa_dest.sin_addr), host))
		return 0;
   
	sa_dest.sin_port = htons(port);

    syslog(6, "[NTP ntp server address %s]", inet_ntoa(sa_dest.sin_addr));

	if (connect(usd, (struct sockaddr *)&sa_dest, sizeof(sa_dest)) == -1) {
		perror("connect");
		return 0;
	}

	return 1;
}

void primary_loop(int usd, int num_probes, int cycle_time)
{
	fd_set fds;
	struct sockaddr sa_xmit;
	int i, pack_len, sa_xmit_len, probes_sent;
	struct timeval to;
	struct ntptime udp_send_ntp;
	int steady_state = 0;

	if (debug) printf("Listening...\n");

	probes_sent = 0;
	sa_xmit_len = sizeof(sa_xmit);
	to.tv_sec = 0;
	to.tv_usec = 0;
	
	for (;;) {
		FD_ZERO(&fds);
		FD_SET(usd, &fds);
		i = select(usd+1, &fds, NULL, NULL, &to);  /* Wait on read or error */
		if ((i != 1) || (!FD_ISSET(usd,&fds))) {
			if (i == EINTR) continue;
			if (i < 0) perror("select");
			
			if ((to.tv_sec == 0) || (to.tv_sec == cycle_time) 
					|| (to.tv_sec == update_interval/*DAY_TIME*/)) {
				if (steady_state != 1 && probes_sent >= num_probes && num_probes != 0) 
					break;
				
				steady_state = 0;
				send_packet(usd, &udp_send_ntp);
				++probes_sent;
				to.tv_sec = cycle_time;
				to.tv_usec = 0;
			}
			continue;
		}
		
		pack_len = recvfrom(usd, incoming, sizeof(incoming), 0, &sa_xmit, &sa_xmit_len);
		
		if (pack_len < 0) {
			perror("recvfrom");
		} else if (pack_len > 0 && pack_len < sizeof(incoming)) {
			steady_state = udp_handle(usd, incoming, pack_len, 
						&sa_xmit, sa_xmit_len, &udp_send_ntp);
		} else {
			printf("Ooops.  pack_len=%d\n", pack_len);
			fflush(stdout);
		}

		if (steady_state == 1) {
			to.tv_sec = update_interval/*DAY_TIME*/;
			to.tv_usec = 0;
		} else if (probes_sent >= num_probes && num_probes != 0) 
			break;
	}
}

static void get_ifname(char *if_name)
{
	char *wan_ifname;

	wan_ifname = nvram_safe_get("wan_ifname");

	strcpy(if_name, wan_ifname);
}

void check_wan_connect(void)
{
	char wan_if[32];
	int fd;
	struct ifreq ifr;
	struct in_addr wan_ip;
	struct timeval to;

	for (;;) {
		memset(&wan_ip, 0, sizeof(struct in_addr));
		get_ifname(wan_if);
		
		if ((fd = socket(AF_INET, SOCK_DGRAM, 0 )) < 0)
			return;

		memset(&ifr, 0, sizeof(ifr));
		strcpy(ifr.ifr_name, wan_if);
		ifr.ifr_addr.sa_family = AF_INET;

		if (ioctl(fd, SIOCGIFADDR, &ifr) == 0)
			wan_ip = ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr;
	
		close(fd);

		if (wan_ip.s_addr == 0) {
			to.tv_sec = 30;
			to.tv_usec = 0;
			select(1, NULL, NULL, NULL, &to); 
		} else
			break;
	}
	
	if (debug) printf("The WAN interface(%s) obtained IP address(%u.%u.%u.%u)!\n",
			wan_if, ((unsigned char *)&wan_ip.s_addr)[0], ((unsigned char *)&wan_ip.s_addr)[1],
			((unsigned char *)&wan_ip.s_addr)[2], ((unsigned char *)&wan_ip.s_addr)[3]);	
}

void usage(char *argv0)
{
	fprintf(stderr,
		"Usage: %s -h hostname [-b second_hostname]\n"
                "\t\t   [-i min_interval -m max_interval (0<=min_interval<=max_interval)]\n"
                "\t\t   [-c count] [-p local_port] [-d] [-s]\n",
		argv0);
}

int main(int argc, char *argv[]) {
	int usd;  /* socket */
	int c;
	/* These parameters are settable from the command line
	   the initializations here provide default behavior */
	short int udp_local_port = 0;   /* default of 0 means kernel chooses */
	int probe_count = 1;            /* default of 0 means loop forever */
	int cycle_time = 15;          /* request timeout in seconds */
	int min_interval = 0;
	int max_interval = 0;
	/* int debug=0; is a global above */
	char *hostname = NULL;          /* must be set */
	char *sec_host = "0.0.0.0";
	char *ntps = "0.0.0.0";
	struct timeval to;

	for (;;) {
		c = getopt(argc, argv, "c:" DST_OPTION TZ_OPTION DEBUG_OPTION NO_DELAY_OPTION INTERVAL_OPTION "h:i:p:b:m:s"); /* yuecheng */
		if (c == EOF) break;
		switch (c) {
			case 'c':
 				probe_count = atoi(optarg);
				break;
#ifdef ENABLE_DEBUG
			case 'd':
				++debug;
				break;
#endif
			case 'h':
				hostname = optarg;
				break;
			case 'i':
				min_interval = atoi(optarg);
				break;
			case 'p':
				udp_local_port = atoi(optarg);
				break;
			case 'b':
				sec_host = optarg;
				break;
			case 'm':
				max_interval = atoi(optarg);
				break;
			case 's':
				set_clock = 1;
				probe_count = 1;
				break;
#if defined(NO_DELAY) /* added by yuecheng */
			case 'u':
				++no_delay;
				break;
#endif
#if defined(SET_TZ)	/* added by lucas */				
			case 'z':
				tz_value = atoi(optarg);
				break;
			case 'x':
				dst_enabled = atoi(optarg);
				break;
			case 't':
				update_interval = atoi(optarg);
				break;		
#endif			/* end by lucas */
			default:
				usage(argv[0]);
				exit(1);
		}
	}

	if (hostname == NULL) {
		usage(argv[0]);
		exit(1);
	}

	if (strcmp(sec_host, "0.0.0.0") == 0)
		sec_host = hostname;

	/* yuecheng */
	if (! no_delay) {
		if (min_interval > max_interval || min_interval < 0 || max_interval < 0) {
			usage(argv[0]);
			exit(1);
		} else if (max_interval == 0) {
			max_interval = cycle_time;
			min_interval = cycle_time;
		} else {
#if NTP_RESTART_PROCESS_TIME
            cycle_time = min_interval;  
#else            
			cycle_time = min_interval + rand()%(max_interval-min_interval+1);
#endif
		}
	} else {
		cycle_time = 1;
	}

	
	if (debug) {
		printf("Configuration:\n"
			"  -c probe_count       %d\n"
			"  -d (debug)           %d\n"
			"  -h hostname          %s\n"
			"  -b second hostname   %s\n"
			"  -i interval(min)     %d\n"
			"  -m interval(max)     %d\n"
			"  -p local_port        %d\n"
			"  -s set_clock         %d\n"
            "  -t update interval   %d\n",
			probe_count, debug, hostname, sec_host, 
			min_interval, max_interval, udp_local_port, set_clock, update_interval);
	}

	check_wan_connect();

	syslog(6, "[NTP start ntpclient]");

	while(1) {
		ntps = (strcmp(ntps, hostname) == 0) ? sec_host : hostname;

		if (debug) {
			printf("Configuration:\n"
				"  Probe count          %d\n"
				"  Debug                %d\n"
				"  NTP server           %s\n"
				"  Interval             %d\n"
				"  Local port           %d\n"
				"  Set clock            %d\n",
				probe_count, debug, ntps, cycle_time, 
				udp_local_port, set_clock);
		}

        syslog(6, "[NTP probe_count=%d hostname=%s update_interval=%d]", probe_count, ntps, update_interval);
		/* Startup sequence */
		if ((usd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
			perror ("socket");
			goto cont;
		}

		if (!setup_receive(usd, INADDR_ANY, udp_local_port)
				|| !setup_transmit(usd, ntps, NTP_PORT)) {
			close(usd);
			goto cont;
		}

		primary_loop(usd, probe_count, cycle_time);
		close(usd);
		goto end;
	cont:	
		to.tv_sec = cycle_time;
		to.tv_usec = 0;
		select(1, NULL, NULL, NULL, &to);
	end:	
		
#if NTP_RESTART_PROCESS_TIME
        cycle_time = min_interval;
#else 
        if ((cycle_time * 2) > max_interval)
			cycle_time = min_interval + rand()%(max_interval-min_interval+1);
		else
			cycle_time = cycle_time * 2;
#endif
	}
	
	return 0;
}

//Brian+ start
struct dstTable {
           unsigned char  startDay[21];
           unsigned char  endDay[21];
           unsigned char  startMonth;
           unsigned char  endMonth;
           unsigned char  diffMonth;
           char           resv;
           int            dstBias;
};

struct dstTable dstEntry[] = {
	{{ }, { }, 0, 0, 0, 0, 0},
		//4 first sun - 10 last sun
	//{{7,6,4,3,2,1,6,5,4,3,1,7,6,5,3,2,1,7,5,4,3}, {27,26,31,30,29,28,26,25,31,30,28,27,26,25,30,29,28,27,25,31,30}, 4, 10, 0, 0, 3600},
	/* DST Start and End date changes beginning March 2007 */
	//3 second sun - 11 first sun
	{{11,9,8,14,13,11,10,9,8,13,12,11,10,8,14,13}, {4,2,1,7,6,4,3,2,1,6,5,4,3,1,7,6}, 3, 11, 0, 0, 3600},
		//3 last sun - 10 last sun
	{{25,30,29,28,27,25,31,30,29,27,26,25,31,29,28,27}, {28,26,25,31,30,28,27,26,25,30,29,28,27,25,31,30}, 3, 10, 0, 0, 3600},
//	{{31,30,28,27,26,25,30,29,28,27,25,31,30,29,27,26,25,31,29,28,27}, {27,26,31,30,29,28,26,25,31,30,28,27,26,25,30,29,28,27,25,31,30}, 3, 10, 0, 0, 3600},
		//10 last sun - 3 last sun
	{{28,26,25,31,30,28,27,26,25,30,29,28,27,25,31,30}, {25,30,29,28,27,25,31,30,29,27,26,25,31,29,28,27}, 10, 3, 1, 0, 3600},
	//for Brazil
	{{14,19,18,17,16,21,20,19,18,16,15,21,20,18,17,16}, {24,16,14,20,19,25,16,15,21,20,18,17,16,15,20,19}, 10, 2, 1, 0, 3600},
	};

struct timeZoneTable_ {
    
           int           gmtOffset;
           char          dstFlag;
};

struct timeZoneTable_ tzEntry[] = 
{
    //{x:gmt+x,0:no DLS 1:NA 2:EU 3:AU} //4:AU changes 3:AU
	{-8,   0},	//0
	{-12,  0},	//1	usys12='(GMT-12:00) Enewetak, Kwajalein'; 
	{-11,  0},	//2	usys13='(GMT-11:00) Midway Island, Samoa';
	{-10,  0},	// usys14='(GMT-10:00) Hawaii';
	{-9,   1},  // usys15='(GMT-09:00) Alaska';
	{-8,   1},	//5	usys16='(GMT-08:00) Pacific Time (US & Canada); Tijuana';
	{-7,   0},	//6	usys17='(GMT-07:00) Arizona ';
	{-7,   1},	// usys18='(GMT-07:00) Mountain Time (US & Canada) ';
	{-6,   1},	// usys19='(GMT-06:00) Central Time (US & Canada) '; 
	{-6,   0},	// usys20='(GMT-06:00) Mexico City, Tegucigalpa ';
	{-6,   1},	//10	usys21='(GMT-06:00) Saskatchewan '; 
	{-5,   0},	//11	usys22='(GMT-05:00) Bogota, Lima, Quito '; 
	{-5,   1},	// usys23='(GMT-05:00) Eastern Time (US & Canada) ';
	{-5,   0},	// usys24='(GMT-05:00) Indiana (East) ';
	{-4,   1},	// usys25='(GMT-04:00) Atlantic Time (Canada) ';
	{-4,   0},	//15	usys26='(GMT-04:00) Caracas, La Paz '; 
	{-4,   1},	//16	usys27='(GMT-04:00) Santiago '; 
	{-3,   1},	// usys28='(GMT-03:00) Newfoundland ';
	{-3,   4},	// usys29='(GMT-03:00) Brasilia ';
	{-3,   0},	// usys30='(GMT-03:00) Buenos Aires, Georgetown ';
	{-2,   0},	//20	usys31='(GMT-02:00) Mid-Atlantic ';
	{-1,   2},	//21	usys32='(GMT-01:00) Azores, Cape Verde Is. ';
	{0,    0},	// usys33='(GMT) Casablanca, Monrovia '; 
	{0,    2},	// usys34='(GMT) Greenwich Mean Time: Dublin, Edinburgh '; 
	{0,    2},	// usys35='(GMT) Greenwich Mean Time: Lisbon, London '; 
	{+1,   2},	//25	usys36='(GMT+01:00) Amsterdam, Berlin, Bern, Rome ';
	{+1,   2},	//26	usys37='(GMT+01:00) Stockholm, Vienna, Belgrade '; 
	{+1,   2},	// usys38='(GMT+01:00) Bratislava, Budapest, Ljubljana '; 
	{+1,   2},	// usys39='(GMT+01:00) Prague,Brussels, Copenhagen, Madrid '; 
	{+1,   2},	// usys40='(GMT+01:00) Paris, Vilnius, Sarajevo, Skopje '; 
	{+1,   2},	//30	usys41='(GMT+01:00) Sofija, Warsaw, Zagreb '; 
	{+2,   2},	//31	usys42='(GMT+02:00) Athens, Istanbul, Minsk ';
	{+2,   2},	// usys43='(GMT+02:00) Bucharest ';
	{+2,   1},	// usys44='(GMT+02:00) Cairo '; 
	{+2,   0},	// usys45='(GMT+02:00) Harare, Pretoria ';
	{+2,   0},	//35	usys46='(GMT+02:00) Helsinki, Riga, Tallinn '; 
	{+2,   2},	//36	usys47='(GMT+02:00) Israel ';
	{+3,   1},	// usys48='(GMT+03:00) Baghdad, Kuwait, Nairobi, Riyadh ';
	{+3,   2},	// usys49='(GMT+03:00) Moscow, St. Petersburg '; 
	{+3,   0},	// usys50='(GMT+03:00) Tehran '; //no match
	{+4,   2},	//40	usys51='(GMT+04:00) Abu Dhabi, Muscat, Tbilisi, Kazan ';
	{+4,   0},	//41	usys52='(GMT+04:00) Volgograd, Kabul '; 
	{+5,   0},	// usys53='(GMT+05:00) Islamabad, Karachi, Ekaterinburg ';
	{+5,   0},	// usys53='(GMT+05:30) Chennai, Kolkata, Mumbai, New Delhig ';
	{+6,   0},	// usys54='(GMT+06:00) Almaty, Dhaka '; 
	{+7,   0},	// usys55='(GMT+07:00) Bangkok, Jakarta, Hanoi ';
	{+8,   0},	//46	usys56='(GMT+08:00) Beijing, Chongqing, Urumqi ';
	{+8,   0},	//47	usys57='(GMT+08:00) Hong Kong, Perth, Singapore, Taipei ';
	{+9,   0},	//	usys58='(GMT+09:00) Toyko, Osaka, Sapporo, Yakutsk ';
	{+9,   3},	// usys58='(GMT+09:30) Toyko, Osaka, Sapporo, Yakutsk ';
	{+10,  3},	// usys59='(GMT+10:00) Brisbane ';
	{+10,  3},	//51	usys60='(GMT+10:00) Canberra, Melbourne, Sydney ';
	//{+10,  0},	//51	usys61='(GMT+10:00) Guam, Port Moresby, Vladivostok ';
	{+10,  3},	//52	usys61='(GMT+10:00) Guam, Port Moresby, Vladivostok ';
	{+10,  3},	//53	usys62='(GMT+10:00) Hobart '; 
	{+11,  0},  // usys63='(GMT+11:00) Magadan, Solamon, New Caledonia ';
	//{+12,  0},  // usys64='(GMT+12:00) Fiji, Kamchatka, Marshall Is. ';
	{+12,  3},	//55	usys65='(GMT+12:00) Wellington, Auckland ';
};

#define date_hash(a, b)   a * 100 + b
int locate_time(struct tm *cur,int mon_s,int mon_e,int day_s,int day_e,int mon_rev) 
{
	int ret_val;
	if (mon_rev==0) 
		ret_val =date_hash((cur->tm_mon+1), cur->tm_mday) >= date_hash(mon_s,day_s) && 
			date_hash((cur->tm_mon+1), cur->tm_mday) <= date_hash(mon_e,day_e) ?  1: 0 ;
	else 
//modified by dvd.chen on 2009/08/05,
//the org code is wrongly dealing the DLS case that start_month > end_month (mon_rev == 1)
#if 0
		ret_val =date_hash((cur->tm_mon+1), cur->tm_mday) <= date_hash(mon_s,day_s) || 
			date_hash((cur->tm_mon+1), cur->tm_mday) >= date_hash(mon_e,day_e) ?  1: 0 ;
#else
    ret_val =date_hash((cur->tm_mon+1), cur->tm_mday) <= date_hash(mon_e,day_e) || 
			date_hash((cur->tm_mon+1), cur->tm_mday) >= date_hash(mon_s,day_s) ?  1: 0 ;
#endif
/*printf("####NTP::cur-M[%d]:D[%d], S-M[%d]:D[%d], E-M[%d]:D[%d]\n", 
    cur->tm_mon, cur->tm_mday, 
    mon_s, day_s, mon_e, day_e);
*/	
	return(ret_val);
}

static void do_DLSTime() //Brian+
{

	int TimeZone;  
	struct tm tm;
	int i,j,ret;
	char  startMonth;
	char  endMonth;
	char  diffMonth;

	struct timeval tv;
	struct timezone tz;
	int TimeZoneValue;
	char buf[32];
	int startDay, endDay;

	//strcpy(buf,nvram_safe_get("time_zone"));
	//TimeZone = atoi(buf);
	TimeZone = tz_value;
	TimeZoneValue = tzEntry[TimeZone].gmtOffset;

	gettimeofday(&tv,&tz);
	//tv.tv_sec = tv.tv_sec+ TimeZoneValue*3600;
	tv.tv_sec = tv.tv_sec+ TimeZoneValue*3600;	
	if ((TimeZone == 49) || (TimeZone == 43)) {	/* +09:30 || +05:30 */
		tv.tv_sec = tv.tv_sec+ 3600/2;		
	}	
	
	settimeofday(&tv,&tz);


	// DayLghtSaving Process start 
	gettimeofday(&tv,&tz);
	memcpy(&tm, localtime(&tv.tv_sec), sizeof(struct tm));
	j=tm.tm_year+1900-2007;
	startMonth = dstEntry[(int)tzEntry[TimeZone].dstFlag].startMonth;
	endMonth = dstEntry[(int)tzEntry[TimeZone].dstFlag].endMonth;
//modified by kenny.wang on 2009/08/11
//change dstTable from 2002~2022 to 2007~2022, it's no need to adjust start and end month  
#if 0
	if ( (tzEntry[TimeZone].dstFlag == 1) && (j <= 4)) {	
					/* NA and before 2007 */
		startMonth += 1;	/* Aprial */
		endMonth -= 1;		/* October */
	}
#endif
	j=tm.tm_year+1900-2007; 
	startDay= dstEntry[(int)tzEntry[TimeZone].dstFlag].startDay[j];
	endDay= dstEntry[(int)tzEntry[TimeZone].dstFlag].endDay[j];
	diffMonth = dstEntry[(int)tzEntry[TimeZone].dstFlag].diffMonth;


	//if(atoi(nvram_safe_get("ntp_dst_enabled")))
	//if(atoi(nvram_get("daylight"))) 
	if (dst_enabled)
	{

		if (tzEntry[TimeZone].dstFlag && locate_time(&tm,startMonth,endMonth,startDay,endDay,diffMonth))
		{

			tv.tv_sec=tv.tv_sec+dstEntry[(int)tzEntry[TimeZone].dstFlag].dstBias;
			settimeofday(&tv,&tz);
			//gettimeofday(&tv,&tz);
			//memcpy(&tm, localtime(&tv.tv_sec), sizeof(struct tm));
		}
	}
	//DayLightSaving Process End	
}
